//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/FreqImp.h,v $
// $Id: FreqImp.h,v 1.5 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_FREQIMPEDANCE_H	// prevent multiple includes
#define ETECH_FREQIMPEDANCE_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Impedance.h>

// --- name space ---

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)


/// represents an frequency dependent electrical impedance base class
/**
 {\tt FreqImpedance} represents an electrical impedance which is depending on 
 the freqency. It is a pure virtual base class serving the implementation
 of capacitors and coils.
 */
class FreqImpedance : public Impedance
{
  public:
    /// constructor
    FreqImpedance(Dbl r = 0.0, Dbl i = 0.0) : Impedance(r,i) {}
    /// constructor
    FreqImpedance(const Cplx &val) : Impedance(val) {}
    /// copy constructor
    FreqImpedance(const FreqImpedance& f) : Impedance(f.Value) {}

    /// destructor
    virtual ~FreqImpedance() {}

    /// overloaded operator () for returning the impedance at frequency {\tt f}
    virtual const Impedance& operator() (Dbl f)
	{ calcVal(f); return *this; }

  protected:
    virtual void calcVal(Dbl f) = 0;
}; // class FreqImpedance


END_NAMESPACE

#endif // ETECH_FREQIMPEDANCE_H
