//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/InvAmp.h,v $
// $Id: InvAmp.h,v 1.6 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_InvAmp_H	// prevent multiple includes
#define ETECH_InvAmp_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Amplifier.h>
#include <AH/Etech/OpAmp.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents an inverting amplifier
/**
 {\tt NoneInvAmp} is an inverting amplifier which uses an
 operational amplifer. It can be used both in DC (direct current) and in 
 AC (alternate current) applications.
 */
class InvAmp: public Amplifier
{
    OpAmp     op;	// used operational amplifier
    Impedance Z1;
    Impedance Z2;

  public:
    /// constructor
    InvAmp(const Impedance& z1, const Impedance& z2) : 
	op(), Z1(z1), Z2(z2) {}
    /// constructor
    InvAmp(const OpAmp& opamp, const Impedance& z1, const Impedance& z2) : 
	op(opamp), Z1(z1), Z2(z2) {}

    /// destructor:
    virtual ~InvAmp() {}

    /// get amplification at frequency {\tt freq}
    virtual Cplx getA(Dbl freq);
    /// get amplification at frequency {\tt freq} with {\tt load}
    virtual Cplx getA(Dbl freq, const Impedance& load);

}; // class InvAmp


END_NAMESPACE

#endif // ETECH_InvAmp_H
