//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/SalKey.h,v $
// $Id: SalKey.h,v 1.3 2001/10/05 23:44:27 ah Exp $
//
// Description: class SalKey
//
//---------------------------------------------------------------------------

#ifndef ETECH_SALKEY_H
#define ETECH_SALKEY_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Impedance.h>
#include <AH/Etech/OpAmp.h>
#include <AH/Etech/Amplifier.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// Sallen-Key Filter
/**
 {\tt SalKey} is a Sallen-Key Filter: a 2-pole filter using an
 operational amplifier.
 */
class SalKey: public Amplifier
{
    OpAmp     op;	// used operational amplifier
    Impedance Z1;	// feedback network
    Impedance Z2;	// feedback network
    Impedance Z3;	// feedback network
    Impedance Z4;	// feedback network
    Impedance Z5;	// feedback network
    Impedance Z6;	// feedback network

  public:
    /// constructors:
    SalKey(const OpAmp& opamp, const Impedance& z1, const Impedance& z2,
	   const Impedance& z3, const Impedance& z4, const Impedance& z5,
	   const Impedance& z6)
	: op(opamp), Z1(z1), Z2(z2), Z3(z3), Z4(z4), Z5(z5), Z6(z6) {}

    /// destructor:
    virtual ~SalKey() {}

    /// get amplification at frequency {\tt freq}:
    virtual Cplx getA(Dbl freq);
    /// get amplification at frequency {\tt freq} with output {\tt load}:
    virtual Cplx getA(Dbl freq, const Impedance& )
	{ return getA(freq); }
};


END_NAMESPACE

#endif /* !ETECH_SALKEY_H */
