//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHetech/IntPol.c,v $
// $Id: IntPol.c,v 1.5 2004/04/20 22:14:57 ah Exp $
//
// Description: implementation of class SpeakerCreator
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: IntPol.c,v 1.5 2004/04/20 22:14:57 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/IntPol.h>
#include <cmath>

// --- local stuff ---

#define TRACE 0
// --- class member implementation ---

// constructor
IntPol::IntPol(IntPolData* data, unsigned int size)
    : table()
{
    for (unsigned int i = 0; i < size; i++)
	table.push_back(data[i]);
#if TRACE
    for (unsigned int i = 0; i < table.size(); i++)
	cerr << i << ": " << table[i].x << " / " << table[i].y << endl;;
#endif
}

// virtual destructor
IntPol::~IntPol()
{
}


Dbl IntPol::getY(Dbl x) const
{
#if TRACE
    cerr << "IntPol::getY(" << x << "): " ;
#endif
    switch (table.size())
    {
    case 0: return 0;
    case 1: return table[0].y;
    default: break;
    }

    if (x < table[0].x)
    {
	// extrapolate using the first two table entries:
	return interpolate(x, table[0], table[1]);
    }

    if (x > table[table.size()-1].x)
    {
	// extrapolate using the last two table entries:
	// interpolate between table[size-1].x and infinity:
	return interpolate(x, table[table.size()-2], table[table.size()-1]);
    }

    // find matching pair in table:
    unsigned int i = 1;
    while (i < table.size())
    {
	if (x < table[i].x) break;
	i++;
    }

    // interpolate between (i-1) and (i)
#if TRACE
    cerr << "3 (i=" << i << "):";
#endif
    return interpolate(x, table[i-1], table[i]);
}

Dbl IntPolLin1Lin2::interpolate
(
    Dbl x, 
    const IntPolData& p,
    const IntPolData& q
) const
{
    Dbl a = (p.y - q.y) / (p.x - q.x);
    return (a * (x - p.x) + p.y);
}

Dbl IntPolLog1Lin2::interpolate
(
    Dbl x, 
    const IntPolData& p,
    const IntPolData& q
) const
{
#if TRACE
    cerr << "interpolate(" << x << ", (" << p.x << "/" << p.y << "), ("
	 << q.x << "/" << q.y << "))" << endl;
#endif
    Dbl a = (p.y - q.y) / log(p.x/q.x);
    return (a * log(x/p.x) + p.y);
}


