//---------------------------------------------------------------------------
// Copyright 2005 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/TestTimeStamp.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: TestTimeStamp.c,v 1.2 2005/02/11 18:52:44 ah Exp $ (c) 2005 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/TimeStamp.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <sstream>
#include <math.h>

USING_NAMESPACE(std)

// --- local defines ---

#define TEST_TIME 1  // set to 0 if necessary

// --- local implementation ---

static bool verbose = false;


static unsigned int test(unsigned int& testnum)
{
    unsigned int errnum = 0;

    // test default constructor
    cout << "Test " << ++testnum << ": ";
    if (verbose) cout << "IOTimeStamp(): ";
    IOTimeStamp ts1;
    unsigned int cl1 = ts1.getUserClocks();
    if (cl1 < 0)
    {
	++errnum;
	if (verbose) cout << "Clocks=" << cl1 << ", expected >= 0: ";
	cout << "failed." << endl;
    }
    else cout << "passed." << endl;

    // test copy constructor
    cout << "Test " << ++testnum << ": ";
    if (verbose) cout << "IOTimeStamp(IOTimeStamp&): ";
    IOTimeStamp ts2 = ts1;
    unsigned int cl2 = ts2.getUserClocks();
    unsigned int eqerr = 0;
    if (cl1 != cl2)
    {
	++eqerr;
	if (verbose) cout << "ucl1=" << cl1 << " != ucl2=" << cl2 << ": ";
    }
    cl1 = ts1.getSystemClocks();
    cl2 = ts2.getSystemClocks();
    if (cl1 != cl2)
    {
	++eqerr;
	if (verbose) cout << "scl1=" << cl1 << " != scl2=" << cl2 << ": ";
    }
    if (eqerr)
    {
	errnum += eqerr;
	cout << "failed." << endl;
    }
    else cout << "passed." << endl;
    if (!eqerr)
    {
	// test operator==
	cout << "Test " << ++testnum << ": ";
	if (verbose) cout << "operator==(IO, IO): ";
	if (ts1 == ts2)
	    cout << "passed." << endl;
	else 
	{
	    ++errnum;
	    if (verbose) cout << "=false, expected=true: ";
	    cout << "failed." << endl;
	}
	// test operator!=
	cout << "Test " << ++testnum << ": ";
	if (verbose) cout << "operator!=(IO, IO): ";
	if (ts1 != ts2)
	{
	    ++errnum;
	    if (verbose) cout << "=false, expected=true: ";
	    cout << "failed." << endl;
	}
	else cout << "passed." << endl;
    }

    // test operator=
    cout << "Test " << ++testnum << ": ";
    if (verbose) cout << "IOTimeStamp::operator=(): ";
    IOTimeStamp ts3;
    ts3 = ts1;
    cl1 = ts1.getUserClocks();
    cl2 = ts3.getUserClocks();
    eqerr = 0;
    if (cl1 != cl2)
    {
	++eqerr;
	if (verbose) cout << "ucl1=" << cl1 << " != ucl2=" << cl2 << ": ";
    }
    cl1 = ts1.getSystemClocks();
    cl2 = ts3.getSystemClocks();
    if (cl1 != cl2)
    {
	++eqerr;
	if (verbose) cout << "scl1=" << cl1 << " != scl2=" << cl2 << ": ";
    }
    if (eqerr)
    {
	errnum += eqerr;
	cout << "failed." << endl;
    }
    else cout << "passed." << endl;

#if TEST_TIME
    // test getUserTime()
    cout << "Test " << ++testnum << ": ";
    if (verbose) cout << "IOTimeStamp::getUserTime(): ";
    // make a loop which needs processing time:
    double tmp = 0;
    for (volatile int i = 0; i < 40000000; ++i)
    {
	tmp += sqrt((double)i);
    }
    IOTimeStamp ts4;
    double t1 = ts1.getUserTime();
    double t2 = ts4.getUserTime();
    double d = t2 - t1;
    if (d <= 0)
    {
	++errnum;
	if (verbose) cout << "t2-t1=" << d << ": expected > 0: ";
	cout << "failed." << endl;
    }
    else
    {
	if (verbose) cout << "t2-t1=" << d << ": ";
	cout << "passed." << endl;
    }

    // test getSystemTime()
    cout << "Test " << ++testnum << ": ";
    if (verbose) cout << "IOTimeStamp::getSystemTime(): ";
    // make a loop which needs processing time:
    for (volatile int i = 0; i < 200000; ++i)
    {
	FILE* fp = fopen("/etc/fstab", "r");
	if (fp)
	{
	    fclose(fp);
	}
    }
    IOTimeStamp ts5;
    t1 = ts1.getSystemTime();
    t2 = ts5.getSystemTime();
    d = t2 - t1;
    if (d <= 0)
    {
	++errnum;
	if (verbose) cout << "t2-t1=" << d << ": expected > 0: ";
	cout << "failed." << endl;
    }
    else
    {
	if (verbose) cout << "t2-t1=" << d << ": ";
	cout << "passed." << endl;
    }
#endif

    return errnum;
}

//----------------------------------------------------------------------
// --- global implementation ---
//----------------------------------------------------------------------


int main(int argc, char* argv[])
{
    cout << "Test of class IOTimeStamp:" << endl;

    if ((2 <= argc) && (0 == strcmp("-v", argv[1])))
	verbose = true;

    unsigned int errnum = 0;
    unsigned int testnum = 0;

    //-------------------------
    // test integer arguments:
    //-------------------------
    errnum += test(testnum);

    //----------------
    // print results:
    //----------------
    if (errnum)
	cout << "Stop. " << errnum << " error" 
	     << (errnum > 1 ? "s":"") << " found in " 
	     << testnum << " tests by " << argv[0] 
	     << "!" << endl;
    else
	cout << "Okay: no errors found in " << testnum << " tests." 
	     << endl;
    return (errnum ? 1 : 0);
}
